<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;

class Albums extends Base {
    protected $table = 'albums';
    public $location;  
    
    public function __construct(\DibiConnection $connection)
    {
        $this->db = $connection;
        $this->location = WWW_DIR."/albums/";
    } 
    
    public function getDataSource($filters=null, $order=null, $datasource=null){
        $datasource = $this->db->dataSource("SELECT A.*,U.username FROM %n A, users U WHERE A.users_id=U.id", $this->table);  
        return parent::getDataSource($filters, $order, $datasource);
    }   
       
    public function getById($id){
        $data = $this->db->fetch("SELECT * FROM %n WHERE id=%i", $this->table, $id);
        if(!$data) return array();
        else return $data;
    }
        
    public function insert($data)
    {
        $this->db->begin();
        $this->db->query("INSERT INTO %n", $this->table, $data);
        $insertid = $this->db->getInsertId();
        if(file_exists($this->location.$insertid))
          Throw new \Exception("Adresář pro album $insertid již existuje! Chyba!"); 
        if (!mkdir($this->location.$insertid, 0777, true)) {
          $this->db->rollback();
          Throw new \Exception("Adresář pro album $insertid nelze vytvořit! Chyba!");
        }   
        $this->db->commit();         
        return $insertid;
    }
    
    public function isAlbumEmpty($albumid) {
      if (!is_readable($this->location.$albumid)) return NULL; 
      return (count(scandir($this->location.$albumid)) == 2);
    }
    
    public function update($data){
        if(!isset($data->id)) return false;
        else {
          $this->db->query("UPDATE %n",$this->table," SET ", $data, " WHERE id=%i", $data->id);
          return true;
        }
    }
    
    public function delete($albumid) {
        if (!is_dir($this->location.$albumid)) {
            throw new \Exception("adresář alba $albumid neexistuje. Nelze operaci provést");
        }
        $path = $this->location.$albumid;
        $this->db->query("DELETE FROM %n WHERE id=%i", $this->table, $albumid);
        array_map('unlink', glob("$path/*.*"));
        rmdir($path);
        return true;
    }
    
    public function isDeleteable($id){
        return $this->db->fetchSingle("SELECT deleteable FROM %n", $this->table, "WHERE id=%i", $id);
    }
    
    public function isEditable($id){
        return $this->db->fetchSingle("SELECT editable FROM %n", $this->table, "WHERE id=%i", $id);
    }    
}
