<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;

class Authorizator extends Base {
    
    protected $userID = NULL;
    public $lastResource="---";
    public $errorMessage;

    public function __construct(\DibiConnection $connection, \Nette\Security\User $user)
    {
        parent::__construct($connection);
        $this->resetErrorMessage();
        $this->userID = $user->getId();  
    }
    
    private function resetErrorMessage(){
        $this->errorMessage="Error nebyl nalezen - kontaktujte programátora";
        return true;
    }
    
    public function getErrorMessage(){
        $message = $this->errorMessage;
        $this->resetErrorMessage();
        return $message;
    }
    
    public function isAllowed($resource="ALL", $privilege="ALL")
    {
        $data = $this->db->fetchAll("SELECT * FROM permissions P, resources R, flags F
                                        WHERE 
                                          P.users_id = %i AND
                                          P.resources_id=R.id AND
                                          P.flags_id=F.id AND
                                          P.disallowed = 0 AND
                                          (R.name=%s OR R.name=%s) AND
                                          (F.name=%s OR F.name=%s)
                                          ", $this->userID, $resource, "ALL", $privilege, "ALL");                               
        if(Count($data)){
           return true;
        }
        else {
          $this->lastResource=$this->db->fetchSingle("SELECT CONCAT(R.description,' - ', F.description) AS lastResource FROM flags F,resources R WHERE F.name=%s AND R.name=%s", $privilege, $resource);
          $this->errorMessage = "Nemáte oprávnění na <strong>".$this->lastResource."</strong>";
          return false;
        }
    } 
    
    public function getResourcesID($resource=NULL)
    {
        return $this->db->fetchSingle("SELECT id FROM resources WHERE name=%s", $resource);
    } 
    
    public function isSuperUser()
    {
        return $this->isAllowed("ALL", "ALL");
    } 
    
    public function isOwner($user, $recordID, $table){
        $ownerID = $this->db->fetchSingle("SELECT users_id FROM %n WHERE id=%i", $table,  $recordID);
        if($user->id == $ownerID)
        {
          return true;
        }  
        else {
          if($this->isSuperUser()) {              
            return true;
          } 
          else { 
            $this->errorMessage = "Nejste majitelem příspěvku #$recordID, a tak s ním nemáte oprávnění manipulovat.";
            return false;
          }
        }
    }
}
