<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;

class Logger extends Base {
    protected $table = 'logger';
    
    const LOG_LEVEL_INFO = 1;
    const LOG_LEVEL_WARNING = 2;
    const LOG_LEVEL_ERROR = 3;
     
    public function __construct(\DibiConnection $connection)
    {
        parent::__construct($connection);
    }
    
    public function LogUserEvent($userID, $resourcesID, $message, $level, $detailid=NULL){
        $data = array(
            "users_id"=>$userID,
            "resources_id"=>$resourcesID,
            "message"=>$message,
            "level"=>$level,
            "time"=>time(),
            "detailid"=>$detailid
        );
        $this->db->query("INSERT INTO %n",$this->table,$data);
        return $this->db->getInsertId();      
    }

    public function LogEvent($resourcesID, $message, $level){
        $data = array(
            "resources_id"=>$resourcesID,
            "message"=>$message,
            "level"=>$level,
            "time"=>time()
        );
        $this->db->query("INSERT INTO logger",$data);
        return $this->db->getInsertId();      
    }
    
    public function GetLogByTime($level=self::LOG_LEVEL_INFO,$since=0){
        if($since==0) $since = strtotime("-7 days");
        return $this->db->fetchAll("SELECT L.*,U.username AS userName
                                      FROM %n L LEFT JOIN users U ON U.id=L.users_id
                                      WHERE level>=%i AND time>=%i ORDER BY id DESC",$this->table,$level, $since);
    }
    
    public function GetLog($level,$limit=30){
        return $this->db->fetchAll("SELECT L.*,U.username AS userName
                                      FROM %n L LEFT JOIN users U ON U.id=L.users_id
                                      WHERE level>=%i ORDER BY id DESC %lmt",$this->table,$level, $limit);
    }
    public function GetLogForResource($resourcesId,$level=self::LOG_LEVEL_WARNING,$limit=30){
        return $this->db->fetchAll("SELECT L.*,U.username AS userName
                                      FROM %n L LEFT JOIN users U ON U.id=L.users_id
                                      WHERE level>=%i AND resources_id=%i ORDER BY id DESC %lmt",$this->table,$level, $resourcesId, $limit);
    }
    
    public function GetLogForResourceAndDetailID($resourcesId,$detailid,$level=self::LOG_LEVEL_INFO,$limit=30){
        return $this->db->fetchAll("SELECT L.*,U.username AS userName
                                      FROM %n L LEFT JOIN users U ON U.id=L.users_id
                                      WHERE level>=%i AND resources_id=%i AND L.detailid=%i ORDER BY id DESC %lmt",$this->table,$level, $resourcesId, $detailid, $limit);
    }
    
    public function getFullLog(){
        return $this->db->dataSource("SELECT L.*,U.username AS userName FROM %n L LEFT JOIN users U ON U.id=L.users_id ORDER BY id DESC",$this->table);
    } 
}
