<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;

class Navigation extends Base {
    protected $table = 'navigation';  
    public $manager = null;
    
    public function __construct(\DibiConnection $connection)
    {
        parent::__construct($connection);
        $this->manager = new NavigationManager($connection);
    } 
    
       
    public function getById($id){
        $data = $this->db->fetch("SELECT * FROM %n WHERE id=%i", $this->table, $id);
        if(!$data) return array();
        else return $data;
    }
        
    public function insert($data)
    {                     
        if($this->db->fetchSingle('SELECT id FROM %n WHERE name LIKE %s', $this->table,$data->name)>0) return false;
        $result = $this->db->query("INSERT INTO %n", $this->table, $data);
        return $this->db->getInsertId();
    }
    
    public function update($data){
        if(!isset($data->id)) return false;
        else {
          $this->db->query("UPDATE %n",$this->table," SET ", $data, " WHERE id=%i", $data->id);
          return true;
        }
    }
    
    public function delete($id){
        $this->db->query("DELETE FROM %n", $this->table, " WHERE id=%i", $id);
    }
    
    public function isDeleteable($id){
        return $this->db->fetchSingle("SELECT deleteable FROM %n", $this->table, "WHERE id=%i", $id);
    }
    
    public function isEditable($id){
        return $this->db->fetchSingle("SELECT editable FROM %n", $this->table, "WHERE id=%i", $id);
    }  
     
}

class NavigationManager extends Base {
    protected $table = "navigations";
        
    public function __construct(\DibiConnection $connection)
    {
        parent::__construct($connection);
    } 
     
    public function getSource($id){
      return $this->db->fetchAll("SELECT * FROM %n WHERE navigation_id=%i ORDER BY parent_id, pos_id", $this->table, $id);
    }
    
    public function getById($id){
        $data = $this->db->fetch("SELECT * FROM %n WHERE id=%i", $this->table, $id);
        if(!$data) return array();
        else return $data;
    }
    
    public function delete($id){
        $curInfo = $this->db->fetch("SELECT * FROM %n WHERE id=%i", $this->table, $id);
        $this->db->query("UPDATE %n SET pos_id=pos_id-1 WHERE navigation_id=%i AND parent_id=%i and pos_id>%i", $this->table, $curInfo->navigation_id, $curInfo->parent_id, $curInfo->pos_id);
        $this->db->query("DELETE FROM %n", $this->table, " WHERE id=%i", $id);
    }
    
    public function update($data){
        if(!isset($data->id)) return false;
        $this->db->query("UPDATE %n",$this->table," SET ", $data, " WHERE id=%i", $data->id);
        return true;
    }
    
    public function insert($data)
    {                     
        $hash = new \Nette\ArrayHash;
        foreach($data as $key=>$value){
          $hash->$key = $value;
        }
        $lastPos = $this->db->fetchSingle('SELECT pos_id FROM %n WHERE navigation_id=%i AND parent_id=%i ORDER BY pos_id DESC LIMIT 1;', $this->table, $data->navigation_id, $data->parent_id);
        $hash->pos_id=0;
        if($lastPos!==false) $hash->pos_id=$lastPos+1;
        $result = $this->db->query("INSERT INTO %n", $this->table, $hash);
        return $this->db->getInsertId();
    }
    
    public function move($navid, $id, $do){
        if(!$this->isValidMove($navid, $id, $do)) return false;
        switch($do){
          case 0:{ // up
            $curInfo = $this->db->fetch('SELECT * FROM %n WHERE id=%i', $this->table, $id);
            $switchID = $this->db->fetchSingle('SELECT id FROM %n WHERE pos_id=%i AND parent_id=%i AND navigation_id=%i', $this->table, $curInfo->pos_id-1, $curInfo->parent_id, $navid);
            $this->db->begin();
            $this->db->query("UPDATE %n SET pos_id=%i WHERE id=%i", $this->table, $curInfo->pos_id, $switchID);
            $this->db->query("UPDATE %n SET pos_id=%i WHERE id=%i", $this->table, $curInfo->pos_id-1, $id);
            $this->db->commit();
            
            break;
          }
          case 1:{ // down
            $curInfo = $this->db->fetch('SELECT * FROM %n WHERE id=%i', $this->table, $id);
            $switchID = $this->db->fetchSingle('SELECT id FROM %n WHERE pos_id=%i AND parent_id=%i AND navigation_id=%i', $this->table, $curInfo->pos_id+1, $curInfo->parent_id, $navid);
            
            $this->db->begin();
            $this->db->query("UPDATE %n SET pos_id=%i WHERE id=%i", $this->table, $curInfo->pos_id, $switchID);
            $this->db->query("UPDATE %n SET pos_id=%i WHERE id=%i", $this->table, $curInfo->pos_id+1, $id);
            $this->db->commit();
            break;
          }
          case 2:{ // left
            
            $curInfo = $this->db->fetch('SELECT * FROM %n WHERE id=%i', $this->table, $id);
            $parInfo = $this->db->fetch('SELECT * FROM %n WHERE id=%i', $this->table, $curInfo->parent_id);
            
            $this->db->begin();
            $this->db->query("UPDATE %n SET pos_id=pos_id+1 WHERE navigation_id=%i AND parent_id=%i and pos_id>%i", $this->table, $navid, $parInfo->parent_id, $parInfo->pos_id);
            $this->db->query("UPDATE %n SET pos_id=%i,parent_id=%i WHERE id=%i", $this->table, $parInfo->pos_id+1, $parInfo->parent_id, $id);
            $this->db->query("UPDATE %n SET pos_id=pos_id-1 WHERE navigation_id=%i AND parent_id=%i and pos_id>%i", $this->table, $navid, $curInfo->parent_id, $curInfo->pos_id);            
            $this->db->commit();
            
            break;
          }
          case 3:{ // right
            $curInfo = $this->db->fetch('SELECT * FROM %n WHERE id=%i', $this->table, $id);
            $parentID = $this->db->fetchSingle("SELECT id FROM %n WHERE navigation_id=%i AND parent_id=%i AND pos_id=%i;", $this->table, $navid, $curInfo->parent_id, $curInfo->pos_id-1);
            $parentInfo = $this->db->fetch('SELECT * FROM %n WHERE id=%i', $this->table, $parentID);
            $lastPos = $this->db->fetchSingle("SELECT pos_id FROM %n WHERE navigation_id=%i AND parent_id=%i ORDER BY pos_id DESC LIMIT 1;", $this->table, $navid, $parentID);
            $newPos=0;
            if($lastPos===false) $newPos = 0;
            else $newPos = $lastPos+1;
            
            $this->db->begin();
            $this->db->query("UPDATE %n SET parent_id=%i, pos_id=%i WHERE id=%i", $this->table, $parentID, $newPos, $id);
            $this->db->query("UPDATE %n SET pos_id=pos_id-1 WHERE navigation_id=%i AND parent_id=%i and pos_id>%i", $this->table, $navid, $curInfo->parent_id, $curInfo->pos_id);
            $this->db->commit();
            
            break;
          }
        }
        return true;
    }
    
    public function isValidMove($navid, $id, $do){
        $source = $this->getSource($navid);
        switch($do){
          case 0:{ // up
            foreach($source as $row){
              if($row->id==$id){
                if($row->pos_id>0) return true;
                else return false;
              }
            }
            break;
          }
          case 1:{ // down
            foreach($source as $row){
              if($row->id==$id){
                $isValid = false;
                    
                foreach($source as $r){
                  if($r->id != $id && $r->parent_id == $row->parent_id){
                    if($r->pos_id>$row->pos_id) $isValid = true;
                  } 
                }
                if($isValid) return true;
                else return false;
              }
            }
            break;          
          }
          case 2:{ // left
            foreach($source as $row){
              if($row->id==$id){
                if($row->parent_id>0) return true;
                else return false;
              }
            }
            break;
          }
          case 3:{ // right
            foreach($source as $row){
              if($row->id==$id){
                if($row->pos_id>0) return true;
                else return false;
              }
            }
            break;
          }
        }
    }
    
}       
     