<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
 
namespace AdminModule;

use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;

class AlbummanagerPresenter extends SecurePresenter
{
    /** @var \UploaderForm @inject */
    public $UploaderForm;
    
    public function startup(){
        parent::startup();
        $this->template->moduleName='Správce Fotogalerie';
        $this->permissionName="ALBUMMANAGER";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
        
    }
    
    /* Action DEFAULT */
    
    public function renderDefault($id) {
        $this->checkPermission("ACCESS"); 
        $this->model->setAlbumID($id);
        
        $this->flashMessage("Na tuto fotogalerii můžete odkazovat za pomocí makra <b>{fotogalerie $id}</b> na Vaší stránce nebo novince.", "info");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect("albums:");
        }   
            
        $this->template->actionName='Nahrávání fotografií';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);    
    }
    
    public function createComponentUploaderForm(){
        $form = $this->UploaderForm->create();    
        $form['files']->addRule(Form::IMAGE, 'Nahrávané fotografie musí být JPEG, PNG nebo GIF.');
        $form->addHidden('albums_id',  $this->getParam('id'));
        $form->onSuccess[] = callback($this, "OnUploadFormResponse");
        return $form;   
    }
    
    public function OnUploadFormResponse(Form $form){
        $this->checkPermission("CREATE");
        $this->checkPermission("EDIT");
        $files = $form->getValues();
        foreach($files->files as $file)
        {          
          $data = new \FileEnchanted();
          $data->uploader = $this->user->id;
          $data->albums_id = $files->albums_id;
          if(isset($files->rewrite) && $files->rewrite==1) $data->rewrite = true;
          else $data->rewrite=false; 
          $data->file = $file;  
          $success = false;
          try {
            $success = $this->model->save($data);
          }
          catch (\Exception $e) {
            $this->flashMessage($e->getMessage(), "danger");
          }
          switch($success){
            case Uploader::SAVE_STATE_ADDED:{
              $this->flashMessage("Fotografie ".$file->name." byla úspěšně nahrána.", "success");
              break;
            }
            case Uploader::SAVE_STATE_REWRITED:{
              $this->flashMessage("Fotografie ".$file->name." byla přepsána.", "warning");
              break;
            }
          }
          
        }
        $this->redirect($this->permissionName.":", $files->albums_id);
    }
    
    /** @var \GridFactory @inject */
    public $GridFactory;
    
    public function createComponentGrid() {         
        $grid = $this->GridFactory->create($this);
        
        $grid->addColumn('id', 'ID souboru')->enableSort();
        $grid->addColumn('time', 'Čas Nahrání');
        $grid->addColumn('file_name', 'Název Souboru');
        $grid->addColumn('title', 'Název fotografie');         
        $grid->addColumn('url', 'URL Adresa');
        $grid->addColumn('username', 'Nahrál');
        
        
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('edit', 'Editovat?');
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');
    
        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');
            $form->addText('username')
              ->setAttribute('class','form-control');             
            return $form;
        });  
            
        return $grid;
    }
    

    
    /* Action NEW */
    
    
    
    /* Action EDIT */
    
    public function renderEdit($id) {
        $this->checkPermission("EDIT");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }  
        
        $this->template->actionName='Editace';
        $this->template->image = "/".$this->model->location."/".$this->model->getAlbumIDByPhotoID($id)."/".$this->model->getFileNameByID($id);
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }
        
    /** @var \FormFactory @inject */
    public $form;
    
    public function createComponentEditForm(){
        $form = $this->form->create();
        $form->addText('title', 'Název:');
        $form->addText('description', 'Krátky Popis:');
        $form->addHidden('id', 'id');
        $form->addSubmit('submit', 'Odeslat'); 
        $form->setDefaults($this->model->getById($this->getParam('id')));
        $form->onSuccess[] = callback($this, "OnEditFormResponse");
        return $form;  
    }
    
    public function OnEditFormResponse($form){
        $this->checkPermission("EDIT");
        $data = $form->getValues();   
        if(empty($data->description)) $data->description=null;
        if(empty($data->title)) $data->title=null;
        try
        {
          $success = $this->model->update($data);
          if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
          else {
            $this->flashMessage("Program zabránil upravení stránky - Stránka neexistuje!", "danger"); 
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistující stránky ".$data->title, Logger::LOG_LEVEL_ERROR);
            return true;
          }
        }
        catch(Exception $e)
        {                                           
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        }
        $this->redirect($this->permissionName.":", $this->model->getAlbumIDByPhotoID($data->id));
        return true;
    }
    
    /* Action DELETE */  
    
    public function handleDelete($id){
        $this->checkPermission("DELETE");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }  
        
        try{        
          if($this->model->delete($id)){
            $this->flashMessage("Soubor byl smazán.", "success");
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání souboru ".$id, Logger::LOG_LEVEL_INFO, $id);
          } else {
            $this->flashMessage("Soubor se nepodařilo smazat!", "danger");
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání souboru ".$id." se nezdařilo", Logger::LOG_LEVEL_ERROR, $id);
          }
        } catch(\Exception $e) {
           $this->flashMessage("Soubor se nepodařilo smazat! [".$e->getMessage()."]", "danger");
           $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání souboru ".$id." se nezdařilo [".$e->getMessage()."]", Logger::LOG_LEVEL_ERROR, $id);
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }  
    
    /* Other Components */
    
    
    
}