<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;


class AlbumsPresenter extends SecurePresenter
{
   
    public function startup(){
        parent::startup();
        $this->template->moduleName='Správa alb';
        $this->permissionName="ALBUMS";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    public function handleDelete($id){
        $this->checkPermission("DELETE");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }              
        
        try{        
          if($this->model->delete($id)){
            $this->flashMessage("Album bylo smazáno.", "success");
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání alba ".$id, Logger::LOG_LEVEL_INFO, $id);
          } else {
            $this->flashMessage("Soubor se nepodařilo smazat!", "danger");
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání alba ".$id." se nezdařilo", Logger::LOG_LEVEL_ERROR, $id);
          }
        } catch(\Exception $e) {
           $this->flashMessage("Soubor se nepodařilo smazat! [".$e->getMessage()."]", "danger");
           $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání alba ".$id." se nezdařilo [".$e->getMessage()."]", Logger::LOG_LEVEL_ERROR, $id);
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }

    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
    
    public function OnNewFormResponse(Form $form){
      $this->checkPermission("CREATE");
      $data = $form->getValues();   
      $data->users_id=$this->user->id;
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $insertID = $this->model->insert($data);
        if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Vytvořeno album - ID ".$insertID." - ".$data->name, Logger::LOG_LEVEL_INFO, $insertID);
        else {
          $this->flashMessage("Program zabránil vytvoření alba - Chyba!", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o vytvoření alba ".$data->name, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(\Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        $this->flashMessage($e->getMessage(), "danger");
      }
      $this->redirect($this->permissionName.":");
      return true;
    }

    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this, true);   
        
        $grid->addColumn('id', 'ID alba');
        $grid->addColumn('name', 'Název alba');
        $grid->addColumn('url', 'URL adresa na album');
        
        $grid->addColumn('username', 'Vlastník');
        
                  
        /* Action*/
        if($this->authorizator->isAllowed("ALBUMMANAGER", "ACCESS")) $grid->addColumn('browseAlbum', 'Spravovat?');
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');
        
        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();  
            $form->addText('name')
              ->setAttribute('class','form-control');      
            return $form;
        });                          
               
        return $grid;
    }
      
    public function createComponentNewForm(){
        $form = new Form;
        $form->addText('name', 'Název alba')
                ->setRequired("Toto políčko je povinné vyplnit")
                ->setAttribute("class", "form-control");
        $form->addSubmit('submit', 'Vytvořit album')
                ->setAttribute("class", "btn btn-primary");
		    $form->onSuccess[] = callback($this, "OnNewFormResponse");
        return $form;   
    }
    
    public function generateURL($id, $action="detail"){
      return parent::generateURL($id, $action);
    }  

}