<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
 
namespace AdminModule;

use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;
use \Nette\Forms\Container;
use \Nette\Forms\Controls\SubmitButton;

class CategoriesPresenter extends SecurePresenter
{
    /** @var \CategoriesNewEditForm @inject */
    public $CategoriesNewEditForm;
    
    public function startup(){
        parent::startup();
        \Kdyby\Replicator\Container::register();
        $this->template->moduleName='Kategorie';
        $this->permissionName="CATEGORIES";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    /* Action DEFAULT */
    
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this, true);
        
        $grid->addColumn('id', 'ID kategorie');
        $grid->addColumn('name', 'Název kategorie');
 
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('edit', 'Editovat?');
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');  
            $form->addText('name')
              ->setAttribute('class','form-control');      
            return $form;
        });    
        
        return $grid;
    }
    
    /* Action NEW */
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
    
    public function createComponentNewForm(){
        $form = $this->CategoriesNewEditForm->create();
		    $form->onSuccess[] = callback($this, "OnNewFormResponse");
        return $form;   
    }
    
    public function newFormAddElementClicked(SubmitButton $button)
    {
      if ($button->parent->isAllFilled()) {
        $button->parent->createOne();
      }
    }

    
    public function OnNewFormResponse($form){
      $this->checkPermission("CREATE");
      $data = $form->getValues();
      try
      {
        $insertID = $this->model->insert($data);
        if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->name, Logger::LOG_LEVEL_INFO, $insertID);
        else {
          $this->flashMessage("Program zabránil vytvoření klíče - Duplikace", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikaci kategorie ".$data->name, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(\Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    /* Action EDIT */
    
    public function renderEdit($id) {
        $this->checkPermission("EDIT");
        $this->template->actionName='Editace';
        $this->flashMessage("Program nedovoluje upravit nadpis této novinky, aby nedošlo k poškození."); 
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }
    
    public function createComponentEditForm(){
        $form = $this->CategoriesNewEditForm->create();
        $form->setDefaults($this->model->getById($this->getParam('id')));
        $form->onSuccess[] = callback($this, "OnEditFormResponse");
        return $form;  
    }
    
    public function OnEditFormResponse($form){
      $this->checkPermission("EDIT");
      $data = $form->getValues(); 
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $success = $this->model->update($data);
        if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
        else {
          $this->flashMessage("Program zabránil upravení novinky - novinka neexistuje!", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistující novinky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    
    /* Action DELETE */  
    
    public function handleDelete($id){
        $this->checkPermission("DELETE");
        $pageInfo = $this->model->getById($id);
        if($pageInfo==array()) {
           $this->flashMessage("Tato novinka nelze smazat - neexistuje!");
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání neexistující novinky ".$id, Logger::LOG_LEVEL_ERROR);
        }
        else {
          $this->model->delete($id);
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání novinky ".$pageInfo['title'], Logger::LOG_LEVEL_INFO, $id);
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }
    
    /* Other Components */
    
    
    
}