<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;


class GlobalsPresenter extends SecurePresenter
{
    /** @var \GlobalsNewEditForm @inject */
    public $GlobalsNewEditForm;
    
    public function startup(){
        parent::startup();
        $this->template->moduleName='Globální proměnné';
        $this->permissionName="GLOBALS";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    public function handleDelete($id){
      $this->checkPermission("DELETE");
      $keyInfo = $this->model->getById($id);
      if($keyInfo==array()) {
         $this->flashMessage("Tento klíč nelze smazat - neexistuje!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání neexistujícího klíče ".$id, Logger::LOG_LEVEL_ERROR);
      }
      else if(!$this->model->isDeleteable($id)) {
        $this->flashMessage("Tento klíč nelze smazat - chráněn!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání nesmazatelného klíče ".$keyInfo['key'], Logger::LOG_LEVEL_ERROR, $id);
      }
      else {
        $this->model->delete($id);
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání klíče ".$keyInfo['key'], Logger::LOG_LEVEL_INFO, $id);
      }
      
      if($this->isAjax()){
        $this->redrawControl();
      }
      else {
        $this->redirect($this->permissionName.":");
      }
    }
   
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
        
    public function renderEdit($id) {
        $this->checkPermission("EDIT");
        $this->template->actionName='Editace';
        if($this->model->isEditable($id)){
          $this->flashMessage("Program nedovoluje upravit název klíče, aby nedošlo k poškození webových stránek."); 
        } else {
          $this->flashMessage("Program nedovoluje tuto globální proměnnou editovat.");
          $this->redirect($this->permissionName.":");
        }         
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }  
    
    public function newForm($form){
      $this->checkPermission("CREATE");
      $data = $form->getValues();   
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $insertID = $this->model->insert($data);
        if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->key, Logger::LOG_LEVEL_INFO, $insertID);
        else {
          $this->flashMessage("Program zabránil vytvoření klíče - Duplikace", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikaci klíče ".$data->key, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    public function OnEditFormResponse($form){
      $this->checkPermission("EDIT");
      $data = $form->getValues();   
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $success = $this->model->update($data);
        if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
        else {
          $this->flashMessage("Program zabránil upravení klíče - Klíč neexistuje!", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistujícího klíče ".$data->key, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
        
    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this);
        
        $grid->addColumn('key', 'Název proměnné');
        $grid->addColumn('value', 'Hodnota proměnné');
        $grid->addColumn('macro', 'Makro');
                  
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('edit', 'Editovat?');
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();  
            $form->addText('key')
              ->setAttribute('class','form-control');      
            return $form;
        });    
        
        return $grid;
    }
      
    public function createComponentNewForm(){
        $form = $this->GlobalsNewEditForm->create();
		    $form->onSuccess[] = callback($this, "newForm");
        return $form;   
    }
    
    public function createComponentEditForm(){
        $form = $this->GlobalsNewEditForm->create();
        $form['key']->setAttribute('readonly', '1');
        $form->setDefaults($this->model->getById($this->getParam('id')));
        $form->onSuccess[] = callback($this, "OnEditFormResponse");
        return $form;  
    }
}