<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;

class NavigationPresenter extends SecurePresenter
{
    private $id;
    
    public function startup(){
        parent::startup();
        $this->template->moduleName='Navigace';
        $this->permissionName="NAVIGATION";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    public function actionDelete($id, $navid){
      $this->checkPermission("DELETE");
      $linkInfo = $this->model->manager->getById($id);
      if($linkInfo==array()) {
         $this->flashMessage("Tato položka nelze smazat - neexistuje!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání neexistující položky ".$id, Logger::LOG_LEVEL_ERROR);
      }
      else {
        $this->model->manager->delete($id);
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání položky ".$linkInfo['content'], Logger::LOG_LEVEL_INFO, $id);
      }
      $this->redirect($this->permissionName.":manage", $navid);
    }
   
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
    
    public function renderNewnav() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
    
    public function renderEdit($id, $navid) {
        $this->checkPermission("EDIT");
        $this->template->actionName='Editace';
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }    
    
    public function renderManage($id) {
        $this->checkPermission("EDIT");
        $this->template->actionName='Správa';
        $this->template->navigationName = $this->model->getById($id)->name;
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
       
        $source = $this->model->manager->getSource($id); 
        $this->id = $id;
        $order = $this->getOrder($source);
        $this->template->component = $this->navRender($source, $order);
        
    } 
       
    
  
    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this, true);
        
        $grid->addColumn('id', 'ID Stránky');
        $grid->addColumn('name', 'Název navigace');
 
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('browse', 'Editovat?');        
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');  
            $form->addText('title')
              ->setAttribute('class','form-control');      
            return $form;
        });    
        
        return $grid;
    }
    
    public function handleMove($id, $act, $itemid){
        $array = array($itemid, $act, $id, "valid"=>$this->model->manager->move($id,$itemid, $act));
        if($this->isAjax()){
              $this->invalidateControl('component');
        } else $this->redirect('manage', array('id'=>$id));
    }
    
    
    /** @var \FormFactory @inject */
    public $FormFactory;
    
    public function createComponentNewnavForm(){
        $form = $this->FormFactory->create();
        $form->addText('name', 'Název')->setAttribute("class", "form-control");      
        $form->addText('root_ul_opening_tag', 'Hlavní otevírací tag pro kořen navigace')
                ->setValue("<ul class='nav navbar-nav'>")->setAttribute("class", "form-control");
        $form->addText('ul_closing_tag', 'Zavírací tag pro navigaci')
                ->setValue("</ul>")->setAttribute("class", "form-control");
        $form->addText('child_ul_opening_tag', 'Hlavní otevírací tag pro podmenu navigace')
                ->setValue("<ul class='dropdown-menu'>")->setAttribute("class", "form-control");
        $form->addText('li_opening_tag', 'Otevírací tag pro <li> bez podmenu:')
                ->setValue("<li>")->setAttribute("class", "form-control");
        $form->addText('li_dropdown_opening_tag', 'Otevírací tag pro <li> s podmenu:')
                ->setValue("<li class='dropdown'>")->setAttribute("class", "form-control");
        $form->addText('li_closing_tag', 'Zavírací tag pro <li>:')
                ->setValue("</li>")->setAttribute("class", "form-control");
        $form->addText('a_opening_tag', 'Otevírací tag odkazu bez podmenu:')
                ->setValue("<a ")->setAttribute("class", "form-control");
        $form->addText('a_dropdown_opening_tag', 'Otevírací tag odkazu s podmenu:')
                ->setValue("<a class='dropdown-toggle' data-toggle='dropdown' role='button' aria-haspopup='true' aria-expanded='false' ")->setAttribute("class", "form-control");
        $form->addText('a_closing_tag', 'Zavírací tag pro odkazy')
                ->setValue("</a>")->setAttribute("class", "form-control");
        
                
        
        
        
        $form->addSubmit('submit', "Přidat")
                ->setAttribute("class", "btn btn-primary");
		    $form->onSuccess[] = callback($this, "onNewNavFormResponse");
        return $form;   
    }
    
    public function onNewNavFormResponse($form){
        $this->checkPermission("CREATE");
        $data = $form->getValues();   
        try
        {
          $insertID = $this->model->insert($data);
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->name, Logger::LOG_LEVEL_INFO, $insertID);
        }
        catch(Exception $e)
        {                                           
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        }
        $this->redirect($this->permissionName.":");     
        return true;
    }       
    
    public function createComponentNewForm(){
        $form = $this->FormFactory->create();
        $form->addHidden("parent_id", $this->getParam('id'));
        $form->addHidden("navigation_id", $this->getParam('navid'));
        $form->addText('content', 'Název');
        $form->addText('href', 'Odkaz (URL):');
        $form->addSubmit('submit', "Přidat");
		    $form->onSuccess[] = callback($this, "onNewFormResponse");
        return $form;   
    }
    
    public function onNewFormResponse($form){
        $this->checkPermission("CREATE");
        $data = $form->getValues();   
        try
        {
          $insertID = $this->model->manager->insert($data);
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->content, Logger::LOG_LEVEL_INFO, $insertID);
        }
        catch(Exception $e)
        {                                           
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        }
        $this->redirect($this->permissionName.":manage", $data->navigation_id);     
        return true;
    }
    
    public function createComponentEditForm(){
        $form = $this->FormFactory->create();
        $form->addHidden("id", $this->getParam('id'));
        $form->addText('content', 'Název');
        $form->addText('href', 'Odkaz (URL):');
        $form->addSubmit('submit', "Updavit");
        $form->setDefaults($this->model->manager->getById($this->getParam('id')));
        $form->onSuccess[] = callback($this, "OnEditFormResponse");
        return $form;  
    }
    
    public function OnEditFormResponse($form){
        $this->checkPermission("EDIT");
        $data = $form->getValues();   
        try
        {
          $success = $this->model->manager->update($data);
          if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
          else {
            $this->flashMessage("Program zabránil upravení položky - Stránka neexistuje!", "danger"); 
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistující položky ".$data->title, Logger::LOG_LEVEL_ERROR);
            return true;
          }
        }
        catch(Exception $e)
        {                                           
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        }
        $this->redirect($this->permissionName.":manage", $this->getParam("navid"));  
        return true;
    }
    
    /*  ----  */
    
  
    
    
    private $toSkip = array();
                
    private function SearchChild($source, $parentID){        
        $node = array();
        foreach($source as $row){                 
          if($row->parent_id==$parentID){
            if(isset($this->toSkip[$parentID])) continue;
            $node[$row->id] = $this->SearchChild($source, $row->id);
            $this->toSkip[$row->id]=true;
          }
        }
        return $node;
    }

    private function renderChild($source, $order, $parentID){   
        $str = "<ul style='list-style-type:none;'>";
        foreach($order as $id=>$row){
           $str.="<li style='padding-left: 30px;'>".$this->getLink($source, $id)."".$this->renderChild($source, $order[$id], $id)."</li>";
        }
        $str .="</ul>";
        return $str;
    }
    
    private function navRender($source, $order){     
        $str = "<ul style='list-style-type:none;font-size: 18px;'>"; 
        $str .="<li><span class='badge badge-success'><a style='color: #FFF;' href=".$this->link('new', array("id"=>0, "navid"=>$this->id))."><i class='fa fa-plus'></i></a></span></li>";
        foreach($order as $id=>$row){
            /*if(!count($row)){
               $str.= "<li>".$this->getLink($source, $id)."</li>";
            } else {*/
               $str.="<li style='padding-left: 30px;'>".$this->getLink($source, $id)."".$this->renderChild($source, $order[$id], $id)."</li>";
            //}
        }
        $str .="</ul>";
        return $str;
    }
    
    private function getLink($source, $id){
        foreach($source as $row){
            if($row->id == $id) {
                $str = "";
                $str.=''.$row['content']." [ ";
                if($row->pos_id>0) $str.="<a class='ajax' href='".$this->link('move!', array("id"=>$this->id,"act"=>0, "itemid"=>$id))."'><i class='fa fa-arrow-up'></i></a> ";
                else $str.="<i class='fa fa-arrow-up' style='opacity: 0.25;'></i> "; 
                
                $isValid = false;
                
                foreach($source as $r){
                  if($r->id != $id && $r->parent_id == $row->parent_id){
                    if($r->pos_id>$row->pos_id) $isValid = true;
                  } 
                }
                
                if($isValid) $str.="<a class='ajax' href='".$this->link('move!', array("id"=>$this->id,"act"=>1, "itemid"=>$id))."'><i class='fa fa-arrow-down'></i></a> ";
                else $str.="<i class='fa fa-arrow-down' style='opacity: 0.25;'></i> "; 
                if($row->parent_id>0) $str.="<a class='ajax' href='".$this->link('move!', array("id"=>$this->id,"act"=>2, "itemid"=>$id))."'><i class='fa fa-arrow-left'></i></a> ";
                else $str.="<i class='fa fa-arrow-left' style='opacity: 0.25;'></i> "; 
                if($row->pos_id>0) $str.="<a class='ajax' href='".$this->link('move!', array("id"=>$this->id,"act"=>3, "itemid"=>$id))."'><i class='fa fa-arrow-right'></i></a>";
                else $str.="<i class='fa fa-arrow-right' style='opacity: 0.25;'></i> "; 
                
                $str.=" <span class='badge badge-success'><a style='color: #FFF;' href=".$this->link('new', array("id"=>$id, "navid"=>$this->id))."><i class='fa fa-plus'></i></a></span>";
                $str.=" <span class='badge badge-danger'><a style='color: #FFF;' href=".$this->link('delete', array("id"=>$id, "navid"=>$this->id))."><i class='fa fa-times'></i></a></span>";
                $str.=" <span class='badge badge-warning'><a style='color: #FFF;' href=".$this->link('edit', array("id"=>$id, "navid"=>$this->id))."><i class='fa fa-edit'></i></a></span>";
                
                $str.=" ]";
                return $str;
            }
        }
    }

    private function getOrder($source){ 
        $node = array();
        foreach($source as $row){
          if(isset($this->toSkip[$row->id])) continue;
          $node[$row->id] = $this->SearchChild($source, $row->id);  
        }
        return $node;
    } 
    
    
        
}
