<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
 
namespace AdminModule;

use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;
use \Nette\Forms\Container;
use \Nette\Forms\Controls\SubmitButton;

class NewsPresenter extends SecurePresenter
{
    /** @var \NewsNewEditForm @inject */
    public $NewsNewEditForm;
    
    public function startup(){
        parent::startup();
        \Kdyby\Replicator\Container::register();
        $this->template->moduleName='Novinky';
        $this->permissionName="NEWS";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    /* Action DEFAULT */
    
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this, true);
        
        $grid->addColumn('id', 'ID novinky');
        $grid->addColumn('visible', '<i class="fa fa-eye"></i> / <i class="fa fa-eye-slash"></i>');
        $grid->addColumn('title', 'Název novinky');
        $grid->addColumn('username', 'Majitel záznamu');
        $grid->addColumn('url', 'URL na novinku');          
        
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('edit', 'Editovat?');
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');  
            $form->addText('title')
              ->setAttribute('class','form-control');      
            return $form;
        });    
        
        return $grid;
    }
    
    /* Action NEW */
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
        
    public function createComponentNewForm(){
        $form = $this->NewsNewEditForm->create();
    
        $categories = $form->addDynamic('categories', function (Container $container) {
            $container->addSelect('categories_id', 'Kategorie:', $this->model->categories->getCategories())
                      ->setAttribute("class", "form-control")
                      ->setAttribute("style", "width: 250px;");
        }, 1);
        
        $categories->addSubmit('add', 'Přidat kategorii')
          ->setAttribute('class', 'ajax')
          ->setValidationScope(FALSE)
          ->onClick[] = callback($this, 'newFormAddElementClicked');

        
		    $form["submit"]->onClick[] = callback($this, "OnNewFormResponse");
        return $form;   
    }
    
    public function newFormAddElementClicked(SubmitButton $button)
    {
        $empty = 0;
        $i = 0;
        $val = $button->parent->getValues();        
        foreach($val as $c){
          $i++;
          if($c->categories_id == 0) {
            $empty++;
            break;
          }
        }
        if (!$empty && $button->parent->isAllFilled()) {
          $button->parent->createOne();
        } 
          
        if($this->isAjax()){
            $this->invalidateControl('newFormDynamic');
        }      
    }  
    
    public function OnNewFormResponse($button){
      $this->checkPermission("CREATE");
      $data = $button->form->getValues();
      $data->users_id = $this->user->id;
      $data->time = time();      
      try
      {
        $insertID = $this->model->insert($data);
        if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->title, Logger::LOG_LEVEL_INFO, $insertID);
        else {
          $this->flashMessage("Program zabránil vytvoření klíče - Duplikace", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikaci novinky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    /* Action EDIT */
    
    public function renderEdit($id) {
        $this->checkPermission("EDIT");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }   
        
        $this->template->actionName='Editace';
        $this->flashMessage("Program nedovoluje upravit nadpis této novinky, aby nedošlo k poškození."); 
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }
    
    public function createComponentEditForm(){
        $news_id = $this->getParam('id');
        $form = $this->NewsNewEditForm->create();
        $form['title']->setAttribute('readonly', '1');
        
        $categories_list = $this->model->getCategoriesByID($news_id);
        $categories_count = Count($categories_list);
        if($categories_count == 0) {$categories_count=1;}
        $categories = $form->addDynamic('categories', function (Container $container) use ($categories_list) {            
            $container->addSelect('categories_id', 'Kategorie:', $this->model->categories->getCategories())
                      ->setAttribute("class", "form-control")
                      ->setAttribute("style", "width: 250px;");
                $id = $container->getName();
                if(isset($categories_list[$id])){
                      $container['categories_id']->setValue($categories_list[$id]['categories_id']);
                }

        }, $categories_count);
        
        
        
        $categories->addSubmit('add', 'Přidat kategorii')
          ->setAttribute('class', 'ajax')
          ->setValidationScope(FALSE)
          ->onClick[] = callback($this, 'editFormAddElementClicked');
   
        
        $form->setDefaults($this->model->getById($news_id));
        $form['submit']->onClick[] = callback($this, "OnEditFormResponse");
        return $form;  
    }
        
    public function editFormAddElementClicked(SubmitButton $button)
    {
        $empty = 0;
        $i = 0;
        $val = $button->parent->getValues();        
        foreach($val as $c){
          $i++;
          if($c->categories_id == 0) {
            $empty++;
            break;
          }
        }
        if (!$empty && $button->parent->isAllFilled()) {
          $button->parent->createOne();
        } 
          
        if($this->isAjax()){
            $this->invalidateControl('editFormDynamic');
        }      
    }
    
    public function OnEditFormResponse($button){
      $this->checkPermission("EDIT");
      $data = $button->form->getValues();
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $success = $this->model->update($data);
        if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
        else {
          $this->flashMessage("Program zabránil upravení novinky - novinka neexistuje!", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistující novinky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    
    /* Action DELETE */  
    
    public function handleDelete($id){
        $this->checkPermission("DELETE");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }  
        
        $pageInfo = $this->model->getById($id);
        if($pageInfo==array()) {
           $this->flashMessage("Tato novinka nelze smazat - neexistuje!");
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání neexistující novinky ".$id, Logger::LOG_LEVEL_ERROR);
        }
        else {
          $this->model->delete($id);
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání novinky ".$pageInfo['title'], Logger::LOG_LEVEL_INFO, $id);
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }
    
    /* Action DELETE */  
    
    public function handleVisible($id){
        $this->checkPermission("EDIT");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }  
        
        $pageInfo = $this->model->getById($id);
        if($pageInfo==array()) {
           $this->flashMessage("Tato novinka nelze editovat - neexistuje!");
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistující novinky ".$id, Logger::LOG_LEVEL_ERROR);
        }
        else {
          $this->model->toggleVisibility($id);
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "editace vititelnosti novinky ".$pageInfo['title'], Logger::LOG_LEVEL_INFO, $id);
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }
    
    /* Other Components */   
    
    public function generateURL($id, $action="read"){
      return parent::generateURL($id, $action);
    }     
}