<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;

class PagesPresenter extends SecurePresenter
{
    /** @var \PagesNewEditForm @inject */
    public $PagesNewEditForm;
    
    public function startup(){
        parent::startup();
        $this->template->moduleName='Stránky';
        $this->permissionName="PAGES";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    public function handleDelete($id){
      $this->checkPermission("DELETE");
      
      if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
      }  
        
      $pageInfo = $this->model->getById($id);
      if($pageInfo==array()) {
         $this->flashMessage("Tato stránka nelze smazat - neexistuje!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání neexistující stránky ".$id, Logger::LOG_LEVEL_ERROR);
      }
      else if(!$this->model->isDeleteable($id)) {
        $this->flashMessage("Tato stránka nelze smazat - chráněn!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání nesmazatelné stránky ".$pageInfo['title'], Logger::LOG_LEVEL_ERROR, $id);
      }
      else {
        $this->model->delete($id);
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání stránky ".$pageInfo['title'], Logger::LOG_LEVEL_INFO, $id);
      }
      
      if($this->isAjax()){
        $this->redrawControl();
      }
      else {
        $this->redirect($this->permissionName.":");
      }
    }
   
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }
        
    public function renderEdit($id) {
        $this->checkPermission("EDIT");
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }             
        
        $this->template->actionName='Editace';
        if($this->model->isEditable($id)){
          $this->flashMessage("Program nedovoluje upravit název této stránky, aby nedošlo k poškození odkazů.");
          $this->flashMessage("Do obsahu stránky lze zapojit <a target='_new' href='http://doc.nette.org/cs/2.3/default-macros'>Latte Makra</a>.<br>
                               Lze využít i makra <span title='Příklad pro globální proměnnou TITLE - {globalVar \"TITLE\"}'>{globalVar \"NAZEV_PROMENNE\"} pro Globální proměnné</span><br>
                               a <span title='Příklad pro navigaci ID 1 - {navigation 1}'>{navigation ID}</span> pro navigaci"); 
        } else {
          $this->flashMessage("Program nedovoluje tuto stránku proměnnou editovat.");
          $this->redirect($this->permissionName.":");
        }
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }  
    
    public function renderDuplicate($id) {
        $this->checkPermission("CREATE");
        $this->template->actionName='Duplikace';
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);
    }
    
    
    public function newForm($form){
      $this->checkPermission("CREATE");
      $data = $form->getValues();   
      
      $imagesSRC = $this->model->getImageSRCFromHTML($data->content);
      foreach($imagesSRC as $image){
        if($this->model->isBase64($image)){
          $fileName = $this->model->saveBase64($image);
          $data->content = str_replace($image, $fileName, $data->content);
        }
      }
      
      $data->users_id = $this->user->id;
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $insertID = $this->model->insert($data);
        if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->title, Logger::LOG_LEVEL_INFO, $insertID);
        else {
          $this->flashMessage("Program zabránil vytvoření stránky - Duplikace. Název stránky musí být unikátní.", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikaci stránky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    public function OnEditFormResponse($form){
      $this->checkPermission("EDIT");
      $data = $form->getValues();   
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $success = $this->model->update($data);
        if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
        else {
          $this->flashMessage("Program zabránil upravení stránky - Stránka neexistuje!", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistující stránky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    public function OnDuplicateFormResponse($form){
      $this->checkPermission("CREATE");
      $data = $form->getValues();
      $data->users_id = $this->user->id;   
      $id = $data->id;
      unset($data->id);
      \Nette\Diagnostics\Debugger::barDump($data);  
      try
      {
        $success = $this->model->duplicate($id, $data);
        if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Duplikován záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $id);
        else {
          $this->flashMessage("Program zabránil duplikování stránky - Stránka neexistuje!", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikování neexistující stránky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(\Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba duplikování - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      catch(\DibiDriverException $e) {
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba duplikování - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
    
    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this, true);
        
        $grid->addColumn('id', 'ID Stránky');
        $grid->addColumn('title', 'Název stránky');
        $grid->addColumn('username', 'Majitel záznamu');
        $grid->addColumn('url', 'URL na stránku');
 
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('edit', 'Editovat?');
        if($this->authorizator->isAllowed($this->permissionName, "CREATE")) $grid->addColumn('duplicate', 'Duplikovat?');         
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');  
            $form->addText('title')
              ->setAttribute('class','form-control');      
            return $form;
        });    
        
        return $grid;
    }
      
    public function createComponentNewForm(){
        $form = $this->PagesNewEditForm->create();
		    $form->onSuccess[] = callback($this, "newForm");
        return $form;   
    }
    
    public function createComponentEditForm(){
        $form = $this->PagesNewEditForm->create();
        $form['title']
                //->setAttribute('readonly', '1')
                  ->setAttribute("class", "form-control")
                  ->setAttribute("style", "width: 250px;");
        $form->setDefaults($this->model->getById($this->getParam('id')));
        $form->onSuccess[] = callback($this, "OnEditFormResponse");
        return $form;  
    }
    
    public function createComponentDuplicateForm(){
        $form = $this->PagesNewEditForm->create(true);
        $form->setDefaults($this->model->getById($this->getParam('id')));
        $form->onSuccess[] = callback($this, "OnDuplicateFormResponse");
        return $form;  
    }
    
}