<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;

class PosterPresenter extends SecurePresenter
{
    /** @var \FormFactory @inject */
    public $FormFactory;
    
    public function startup(){
        parent::startup();
        $this->template->moduleName='Plakáty';
        $this->permissionName="POSTER";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
    
    
    public function handleVisible($id) {
        $this->checkPermission("EDIT");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        }  
        
        $data = $this->model->getById($id);
        if($data==array()) {
           $this->flashMessage("Tento záznam nelze editovat - neexistuje!");
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistujícího záznamu ".$id, Logger::LOG_LEVEL_ERROR);
        }
        else {
            $this->model->toggleVisibility($id);
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "editace vititelnosti záznamu ".$data['title'], Logger::LOG_LEVEL_INFO, $id);                                                            
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }
    
    
    public function handleDelete($id){
      $this->checkPermission("DELETE");
      
      if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
      }  
        
      $pageInfo = $this->model->getById($id);
      if($pageInfo==array()) {
         $this->flashMessage("Tento záznam nelze smazat - neexistuje!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání neexistujícího záznamu ".$id, Logger::LOG_LEVEL_ERROR);
      }
      else if(!$this->model->isDeleteable($id)) {
        $this->flashMessage("Tato stránka nelze smazat - chráněn!");
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o smazání nesmazatelné záznamu ".$pageInfo['title'], Logger::LOG_LEVEL_ERROR, $id);
      }
      else {
        $this->model->delete($id);
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání záznamu ".$pageInfo['title'], Logger::LOG_LEVEL_INFO, $id);
      }
      
      if($this->isAjax()){
        $this->redrawControl();
      }
      else {
        $this->redirect($this->permissionName.":");
      }
    }
   
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    public function renderNew() {
        $this->checkPermission("CREATE");
        $this->template->actionName='Přidání';
    }

    
    public function newForm($form){
      $this->checkPermission("CREATE");
      $data = $form->getValues();      
      $file = $data->file;
      unset($data->file);        
      $data->users_id = $this->user->id;
      $data->time = time();
      try
      {
        $insertID = $this->model->insert($data);
        if($file->isOk()){
            $file->move(WWW_DIR."/poster/".$insertID.".jpg");        
        }
        if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ID ".$insertID." - ".$data->title, Logger::LOG_LEVEL_INFO, $insertID);
        else {
          $this->flashMessage("Program zabránil vytvoření plakátu - Duplikace. Název plakátu musí být unikátní.", "danger"); 
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikaci stránky ".$data->title, Logger::LOG_LEVEL_ERROR);
          return true;
        }
      }
      catch(Exception $e)
      {                                           
        $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
      }
      $this->redirect($this->permissionName.":");
      return true;
    }
       
    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this, true);
                                                     
        $grid->addColumn('id', 'ID Plakátu');
        $grid->addColumn('visible', '<i class="fa fa-eye"></i> / <i class="fa fa-eye-slash"></i>');
        $grid->addColumn('title', 'Název plakátu');
        $grid->addColumn('username', 'Majitel záznamu'); 
        /* Action*/    
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');  
            $form->addText('title')
              ->setAttribute('class','form-control');      
            return $form;
        });    
        
        return $grid;
    }
      
    public function createComponentNewForm(){
        $form = $this->FormFactory->create();
        $form->addUpload("file", "Soubor s plakátem")
             ->setRequired("Je třeba vybrat plakát k nahrání");          
        $form->addText("title", "Titulek")
              ->setRequired("Je třeba vybrat plakát k nahrání")
              ->addRule(Form::MAX_LENGTH, 'Poznámka je příliš dlouhá (%i znaků maximálně)', 64);                 
        $form->addRadioList("visible", "Má se plakát zobrazovat?", array(0=>"NE", 1=>"ANO"));
        $form->addSubmit("submit", "Odeslat");
		    $form->onSuccess[] = callback($this, "newForm");
        return $form;   
    }
    
}