<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;


class UploaderPresenter extends SecurePresenter
{
    /** @var \UploaderForm @inject */
    public $UploaderForm;
        
    public function handleDelete($id){
        $this->checkPermission("DELETE");
        
        if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $id, $this->model->getTable())) {
          $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
          $this->redirect($this->permissionName.":");
        } 
        
        try{        
          if($this->model->delete($id)){
            $this->flashMessage("Soubor byl smazán.", "success");
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání souboru ".$id, Logger::LOG_LEVEL_INFO, $id);
          } else {
            $this->flashMessage("Soubor se nepodařilo smazat!", "danger");
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání souboru ".$id." se nezdařilo", Logger::LOG_LEVEL_ERROR, $id);
          }
        } catch(\Exception $e) {
           $this->flashMessage("Soubor se nepodařilo smazat! [".$e->getMessage()."]", "danger");
           $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání souboru ".$id." se nezdařilo [".$e->getMessage()."]", Logger::LOG_LEVEL_ERROR, $id);
        }
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }
    }     
    
    public function startup(){
        parent::startup();
        $this->template->moduleName='Nahrané Soubory';
        $this->permissionName="UPLOADER";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
        
    }
    
    public function OnUploadFormResponse(Form $form){
        $this->checkPermission("CREATE");
        $this->checkPermission("EDIT");
        $files = $form->getValues();
        foreach($files->files as $file)
        {          
          $data = new \FileEnchanted();
          $data->uploader = $this->user->id;
          if(isset($files->rewrite) && $files->rewrite==1) $data->rewrite = true;
          else $data->rewrite=false; 
          $data->file = $file;  
          $success = false;
          try {
            if($data->rewrite==true){
              if($this->model->exists($data)){
                if(CHECK_OWNERSHIP==true && !$this->authorizator->isOwner($this->user, $this->model->getFileID($data), $this->model->getTable())) {
                    $this->flashMessage($this->authorizator->getErrorMessage(), "danger");
                } 
                else {
                  $success = $this->model->save($data);
                }
              }
            }
            else {
              $success = $this->model->save($data);
            }
          }
          catch (\Exception $e) {
            $this->flashMessage($e->getMessage(), "danger");
          }
          switch($success){
            case Uploader::SAVE_STATE_ADDED:{
              $this->flashMessage("Soubor ".$file->name." byl úspěšně nahrán.", "success");
              break;
            }
            case Uploader::SAVE_STATE_REWRITED:{      
              $this->flashMessage("Soubor ".$file->name." byl přepsán.", "warning");
              break;
            }
          }
          
        }
        $this->redirect($this->permissionName.":");
    }
   
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Nahrávání souborů';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);    
    }
      
    public function createComponentUploaderForm(){
        $form = $this->UploaderForm->create();
        $form->onSuccess[] = callback($this, "OnUploadFormResponse");
        return $form;   
    }
    
    /** @var \GridFactory @inject */
    public $GridFactory;
    
    public function createComponentGrid() {         
        $grid = $this->GridFactory->create($this);
        
        $grid->addColumn('id', 'ID souboru')->enableSort();
        $grid->addColumn('time', 'Čas Nahrání');
        $grid->addColumn('file_name', 'Název Souboru');
        $grid->addColumn('url', 'URL Adresa');
        $grid->addColumn('username', 'Nahrál');
        
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');
    
        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');
            $form->addText('username')
              ->setAttribute('class','form-control');
            $form->addText('email')
              ->setType('email')
              ->setAttribute('class','form-control');              
            return $form;
        });   
                
        return $grid;
    }
    
}