<?php
/*
 * 
The MIT License (MIT)

Copyright (c) 2014 Lukáš Zelenka [Xchos]
 * 
 */
namespace AdminModule;
use Nette\Application\UI\Form,
    Nette\Utils\Strings,
    Tracy\Debugger;


class UsersPresenter extends SecurePresenter
{
    public function handleDelete($id)
    {
        $this->checkPermission("DELETE");
        if($this->model->delete($id)) {
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání uživatele $id", Logger::LOG_LEVEL_INFO, $id);
        }
        else {
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Smazání uživatele $id se nezdařilo", Logger::LOG_LEVEL_ERROR, $id);
        } 
        
        if($this->isAjax()){
          $this->redrawControl();
        }
        else {
          $this->redirect($this->permissionName.":");
        }                             
    }

    public function startup(){
        parent::startup();
        $this->template->moduleName='Uživatelské účty';
        $this->permissionName="USERS";
        $this->resourcesId=$this->authorizator->GetresourcesId($this->permissionName);
    }
   
    public function renderDefault() {
        $this->checkPermission("ACCESS");      
        $this->template->actionName='Přehled';
        $this->template->logs=$this->logger->GetLogForResource($this->resourcesId, Logger::LOG_LEVEL_INFO);
    }
    
    public function renderNew() {
        $this->checkPermission("CREATE");   
        $this->template->actionName='Přidání';
    }
        
    public function renderEdit($id) {
        if($this->user->id!=$id){ // Editace vlastního účtu není třeba kontrolovat - změna hesla
          $this->checkPermission("EDIT");
        }      
        $this->flashMessage("Program nedovoluje upravit název klíče, aby nedošlo k poškození webových stránek."); 
        $this->template->actionName='Editace';
        $this->template->logs = $this->logger->GetLogForResourceAndDetailID($this->resourcesId, $id);        
    }     
    
    public function newForm($form){
        $this->checkPermission("CREATE");
        $data = $form->getValues();
        try
        {
          $insertID = $this->model->insert($data);
          if($insertID>0) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Přidán nový záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $insertID);
          else {
            $this->flashMessage("Program zabránil vytvoření uživatele - Duplikace", "danger"); 
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o duplikaci uživatele ".$data->key, Logger::LOG_LEVEL_ERROR);
            return true;
          }
        }
        catch(Exception $e)
        {                                           
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba přidání - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        }
        $this->redirect($this->permissionName.":");
        return true;
    }
  
    public function OnEditFormResponse($form){
        $data = $form->getValues();   
        if($this->user->id!=$data->id){ // Editace vlastního účtu není třeba kontrolovat - změna hesla
          $this->checkPermission("EDIT");
        }      
        try
        {
          $success = $this->model->update($data);
          if($success) $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Upraven záznam - ".$this->getName().":".$this->getAction(), Logger::LOG_LEVEL_INFO, $data->id);
          else {
            $this->flashMessage("Program zabránil upravení uživatele - Uživatel neexistuje!", "danger"); 
            $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Pokus o editaci neexistujícího uživatele ".$data->username, Logger::LOG_LEVEL_ERROR);
            return true;
          }
        }
        catch(Exception $e)
        {                                           
          $this->logger->logUserEvent($this->user->id, $this->resourcesId, "Chyba editace - ".$this->getName().":".$this->getAction()." - ".$e->getMessage(), Logger::LOG_LEVEL_ERROR);
        }
        $this->redirect($this->permissionName.":");
        return true;
    }

    /** @var \GridFactory @inject */
    public $GridFactory;
      
    public function createComponentGrid() {
        $grid = $this->GridFactory->create($this);
        
        $grid->addColumn('id', 'ID Uživatele');
        $grid->addColumn('username', 'Uživatelské jméno');
        $grid->addColumn('email', 'E-mailová adresa');
        $grid->addColumn('time', 'Poslední přihlášení');
        $grid->addColumn('login_count', 'Počet přihlášení');
        $grid->addColumn('last_ip', 'Posledně z IP adresy');
                  
        /* Action*/
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('edit', 'Editovat?');
        if($this->authorizator->isAllowed($this->permissionName, "EDIT")) $grid->addColumn('permissions', 'Práva uživatele');
        if($this->authorizator->isAllowed($this->permissionName, "DELETE")) $grid->addColumn('delete', 'Smazat?');

        $self = $this; // required for php < 5.4
        $grid->setFilterFormFactory(function() use($self) {
            $form = $self->GridFactory->createFilterForm();
            $form->addText('id')
              ->setType('number')
              ->setAttribute('class','form-control');
            $form->addText('username')
              ->setAttribute('class','form-control');
            $form->addText('email')
              ->setType('email')
              ->setAttribute('class','form-control');             
            return $form;
        });    
        
        return $grid;
    }

    private function createForm(){
        $form = new Form;
        $form->addText('email', 'Email:')->addRule(Form::EMAIL, 'Zadejte email')->addRule(Form::FILLED, 'Zadejte email');;
        $form->addText('username', 'Uživatelské jméno')->addRule(Form::FILLED, 'Zadejte Uživatelské jméno');
        $form->addPassword('password', "Heslo:")->addRule(Form::FILLED, 'Zadejte heslo');
        $form->addHidden('id', 'id');
        return $form;
    }  
         
    public function createComponentNewForm(){
        $form = $this->createForm();
        $form->onSuccess[] = callback($this, "newForm");
        $form->addSubmit('submit', 'Přidat'); 
        return $form;    
    }
    
    public function createComponentEditForm(){
        $form = $this->createForm();
        $form->onSuccess[] = callback($this, "OnEditFormResponse");
        $form->setDefaults($this->model->getById($this->getParam('id')));
        $form->addSubmit('submit', 'Upravit');
        return $form;
    }
}
