<?php

namespace Presenters;

use Nette;


abstract class BasePresenter extends Nette\Application\UI\Presenter
{
  /** @inject @var \DibiConnection */
  public $db;
  
  public $model; 
  
  /** @inject @var \Mobile_Detect */
  public $mobileDetect;
  
  protected function startup()
  {
      parent::startup();    
        
      // Automatic $this->model instance creator
      $model = $this->ModelNameByClass(get_class($this)); // Gets model name from presenter Name
                   
      if(class_exists($model)){
        $this->model = new $model($this->db);
      }
      
      $this->template->mobileDetect = $this->mobileDetect;
      
      $this->template->user = $user = \Nette\Environment::getUser();
      $this->template->logged = $user->isLoggedIn() ? $user->getIdentity()->username : NULL;
      
      $this->template->title = "Zimní stadion Rakovník";
           
  }
   
  private function ModelNameByClass($className, $strip=false){
      if($strip==true) $start = strpos($className, "\\")+1;
      else $start = 0;
      return substr($className, $start, -strlen("Presenter")); 
  }
  
  protected function createTemplate()
  {
      $template = parent::createTemplate();
      $latte = $template->getLatte();
      $compiler = $latte->getCompiler();
      $macros = new \Nette\Latte\Macros\MacroSet($compiler);
      
      $macros->addMacro('globalVar', 'echo \FrontModule\globals::getByName(%node.args)');
      $macros->addMacro('widget', 'echo \FrontModule\widgets::getByID(%node.args)');
      $macros->addMacro('navigation', 'echo \FrontModule\navigations::getByID(%node.args)');
      $macros->addMacro('since', 'echo \FrontModule\base::since(%node.args)');
      $macros->addMacro('fileURL', 'echo \FrontModule\files::getByID(%node.args)');
      
      $macros->addMacro('fancy', 'echo \FrontModule\files::generateFancy(%node.args)', 'echo "</a>"');
      $macros->addMacro('obrazek', 'echo \FrontModule\files::generateImage(%node.args)');
      
      
      $macros->addMacro('fotogalerie', 'echo \FrontModule\AlbumsPresenter::createAlbumWidget(%node.args)');
      
      
      $macros->addMacro('spacer', 'echo \FrontModule\base::generateSpacer(%node.args)');
      return $template;
  }
  
  public function prepareToRender($var, $varName="unnamed"){
    $presenterName = $this->ModelNameByClass(get_class($this), true);
    $actionName = $this->getAction();
    $tempDir = WWW_DIR."/../temp/";
    $preRenderDir = "preRender";
    if(!is_dir($tempDir.$preRenderDir)) mkdir($tempDir.$preRenderDir);
    $path = $tempDir.$preRenderDir."/".$presenterName.".".$actionName.".".$varName.".latte";    
    file_put_contents($path, $var);  
    return $path;
  }
  
  
}
